@@ -61,6 +61,8 @@ class MDRenderer:
         *,
         finalize: bool = True,
     ) -> str:
+        self._prepare_env(env)
+
         # Update RENDERER_MAP defaults with renderer functions defined
         # by plugins.
         updated_renderers = {}
@@ -81,20 +83,23 @@ class MDRenderer:
                     postprocessors[syntax_name] += (pp,)
         renderer_map = MappingProxyType({**DEFAULT_RENDERERS, **updated_renderers})
         postprocessor_map = MappingProxyType(postprocessors)
+
         render_context = RenderContext(renderer_map, postprocessor_map, options, env)
         text = tree.render(render_context)
         if finalize:
-            if env.get("used_refs"):
+            if env["used_refs"]:
                 text += "\n\n"
                 text += self._write_references(env)
             if text:
                 text += "\n"
+
+        assert "\x00" not in text, "null bytes should be removed by now"
         return text
 
     @staticmethod
     def _write_references(env: MutableMapping) -> str:
         ref_list = []
-        for label in sorted(env.get("used_refs", [])):
+        for label in sorted(env["used_refs"]):
             ref = env["references"][label]
             destination = ref["href"] if ref["href"] else "<>"
             destination = unescape_string(destination)
@@ -105,3 +110,7 @@ class MDRenderer:
                 item += f' "{title}"'
             ref_list.append(item)
         return "\n".join(ref_list)
+
+    def _prepare_env(self, env: MutableMapping) -> None:
+        env["indent_width"] = 0
+        env["used_refs"] = set()
